@@ -1,20 +1,14 @@
-"""
-    flask_user
-    ----------
-    Flask-User is a customizable user management extension for Flask.
+""" Flask-User is a customizable user account management extension for Flask.
 
     :copyright: (c) 2013 by Ling Thio
     :author: Ling Thio (ling.thio@gmail.com)
-    :license: Simplified BSD License, see LICENSE.txt for more details.
-"""
+    :license: Simplified BSD License, see LICENSE.txt for more details."""
 
 from passlib.context import CryptContext
-
 from flask import Blueprint, current_app
 from flask_login import LoginManager, UserMixin as LoginUserMixin
-from flask_user.db_interfaces import DBInterface
-
-from .db_interfaces import SQLAlchemyAdapter
+from flask_user.db_adapters import DBAdapter
+from .db_adapters import SQLAlchemyAdapter
 from . import forms
 from . import passwords
 from . import settings
@@ -22,7 +16,7 @@ from . import tokens
 from . import translations
 from . import views
 
-__version__ = '0.4.1'
+__version__ = '0.4.2'
 
 # Enable the following: from flask.ext.user import current_user
 from flask_login import current_user
@@ -31,23 +25,16 @@ from flask_login import current_user
 from .decorators import *
 
 def _user_loader(user_id):
-    """
-    Flask-Login helper function to load user by user_id
-    """
+    """ Flask-Login helper function to load user by user_id"""
     um = current_app.user_manager
-    return um.db_adapter.find_user_by_id(user_id=user_id)
+    return um.find_user_by_id(user_id)
 
 def _flask_user_context_processor():
-    """
-    Make 'user_manager' available to Jinja2 templates
-    """
+    """ Make 'user_manager' available to Jinja2 templates"""
     return dict(user_manager=current_app.user_manager)
 
-
 class UserManager(object):
-    """
-    This is the Flask-User object that manages the User management process.
-    """
+    """ This is the Flask-User object that manages the User management process."""
 
     def __init__(self, db_adapter, app=None,
                 # Forms
@@ -77,9 +64,7 @@ class UserManager(object):
                 token_manager=tokens.TokenManager(),
                 password_crypt_context=None,
                 ):
-        """
-        Initialize the UserManager with custom or built-in attributes
-        """
+        """ Initialize the UserManager with custom or built-in attributes"""
         self.db_adapter = db_adapter
         self.lm = login_manager
         # Forms
@@ -113,9 +98,7 @@ class UserManager(object):
             self.init_app(app)
 
     def init_app(self, app):
-        """
-        Initialize app.user_manager.
-        """
+        """ Initialize app.user_manager."""
         # Bind Flask-USER to app
         app.user_manager = self
 
@@ -130,8 +113,7 @@ class UserManager(object):
             app.jinja_env.install_gettext_callables(
                     translations.gettext,
                     translations.ngettext,
-                    newstyle=True
-                    )
+                    newstyle=True)
 
         # Create password_crypt_context if needed
         if not self.password_crypt_context:
@@ -162,7 +144,7 @@ class UserManager(object):
         self.lm.init_app(app)
 
     def add_url_routes(self, app):
-        # Add URL Routes
+        """ Add URL Routes"""
         if self.enable_confirm_email:
             app.add_url_rule(self.confirm_email_url, 'user.confirm_email', self.confirm_email_view_function)
             app.add_url_rule(self.resend_confirm_email_url, 'user.resend_confirm_email', self.resend_confirm_email_view_function)
@@ -177,9 +159,8 @@ class UserManager(object):
         app.add_url_rule(self.logout_url, 'user.logout', self.logout_view_function, methods=['GET', 'POST'])
         if self.enable_register:
             app.add_url_rule(self.register_url, 'user.register', self.register_view_function, methods=['GET', 'POST'])
-        # We can not define 'user.unauthenticated' here because it clashes with 'user.login'
-        # We can not define 'user.unauthorized' here because it clashes with 'home_page'
 
+    # Obsoleted function. Replace with hash_password()
     def generate_password_hash(self, password):
         return passwords.hash_password(self, password)
 
@@ -195,30 +176,51 @@ class UserManager(object):
     def verify_token(self, token, expiration_in_seconds):
         return self.token_manager.verify_token(token, expiration_in_seconds)
 
+    def find_user_by_id(self, user_id):
+        return self.db_adapter.find_object(self.db_adapter.UserClass, id=user_id)
+
+    def find_user_by_username(self, username):
+        return self.db_adapter.ifind_object(self.db_adapter.UserClass, username=username)
+
+    def find_user_by_email(self, email):
+        return self.db_adapter.ifind_object(self.db_adapter.UserClass, email=email)
+
+    def email_is_available(self, new_email):
+        """ Return True if new_email does not exist.
+            Return False otherwise."""
+        return self.find_user_by_email(new_email)==None
+
+    def username_is_available(self, new_username):
+        """ Return True if new_username does not exist or if new_username equals old_username.
+            Return False otherwise."""
+        # Allow user to change username to the current username
+        if current_user.is_authenticated() and new_username == current_user.username:
+            return True
+        # See if new_username is available
+        return self.find_user_by_username(new_username)==None
+
+
+
 class UserMixin(LoginUserMixin):
-    """
-    This class adds methods to the User model class required by Flask-Login and Flask-User.
-    """
+    """ This class adds methods to the User model class required by Flask-Login and Flask-User."""
     
     def has_roles(self, *requirements):
-        """
-        Return True if the user has all of the specified roles. Return False otherwise.
-        
-        has_roles() accepts a list of requirements:
-            has_role(requirement1, requirement2, requirement3).
-            
-        Each requirement is either a role_name, or a tuple_of_role_names.
-            role_name example:   'manager'
-            tuple_of_role_names: ('funny', 'witty', 'hilarious')
-        A role_name-requirement is accepted when the user has this role.
-        A tuple_of_role_names-requirement is accepted when the user has ONE of these roles.
-        has_roles() returns true if ALL of the requirements have been accepted.
-         
-        For example:
-            has_roles('a', ('b', 'c'), d)
-        Translates to:
-            User has role 'a' AND (role 'b' OR role 'c') AND role 'd'
-        """
+        """ Return True if the user has all of the specified roles. Return False otherwise.
+
+            has_roles() accepts a list of requirements:
+                has_role(requirement1, requirement2, requirement3).
+
+            Each requirement is either a role_name, or a tuple_of_role_names.
+                role_name example:   'manager'
+                tuple_of_role_names: ('funny', 'witty', 'hilarious')
+            A role_name-requirement is accepted when the user has this role.
+            A tuple_of_role_names-requirement is accepted when the user has ONE of these roles.
+            has_roles() returns true if ALL of the requirements have been accepted.
+
+            For example:
+                has_roles('a', ('b', 'c'), d)
+            Translates to:
+                User has role 'a' AND (role 'b' OR role 'c') AND role 'd'"""
 
         # Translates a list of role objects to a list of role_names
         user_roles = [role.name for role in self.roles]
